@@ -6,12 +6,16 @@ from collections import defaultdict
 import requests
 import lxml.etree as ET
 
-from . string_utils import clean_markup
+from freud_api_crawler.string_utils import clean_markup, extract_page_nr
+
+from freud_api_crawler.tei_utils import make_pb
 
 
 FRD_API = os.environ.get('FRD_API', 'https://www.freud-edition.net/jsonapi/')
 FRD_USER = os.environ.get('FRD_USER', False)
 FRD_PW = os.environ.get('FRD_PW', False)
+FRD_SESSION = os.environ.get('FRD_SESSION', False)
+FRD_TOKEN = os.environ.get('FRD_TOKEN', False)
 
 SAMPLE_MANIFESTATION = os.path.join(
     os.path.dirname(__file__),
@@ -30,6 +34,8 @@ TEI_DUMMY = os.path.join(
     "tei_dummy.xml"
 )
 
+CUR_LOC = os.path.dirname(os.path.abspath(__file__))
+
 
 class FrdClient():
 
@@ -45,7 +51,11 @@ class FrdClient():
         """
         if self.authenticated:
             r = requests.get(
-                self.endpoint, auth=(self.user, self.pw)
+                self.endpoint,
+                auth=(self.user, self.pw),
+                headers={
+                    'Cookie': self.cookie
+                }
             )
             result = r.json()
             d = defaultdict(list)
@@ -57,47 +67,17 @@ class FrdClient():
         else:
             return {}
 
-    # def crawl_endpoint(self, url_part, important_values):
-    #     """ method for iterating over drupal-endpoints
-    #     :param url_part: specific API endpoint, e.g. 'werk' -> {drupalbase}/jsonapi/node/{url_part}
-    #     :type gnd_id: str
-    #     :param important_values: simple attribute names, e.g. ['title', 'field_year']
-    #     :type user: list
-    #
-    #     :return: print things
-    #     """
-    #
-    #     url = f"{self.endpoint}node/{url_part}?page[limit]={self.page_size}"
-    #     next_page = True
-    #     counter = 0
-    #     auth = ()
-    #     while next_page:
-    #         print(url)
-    #         response = requests.get(url, auth=(self.user, self.pw))
-    #         result = response.json()
-    #         links = result['links']
-    #         for y in result['data']:
-    #             self_uri = y['links']['self']['href']
-    #             print(self_uri)
-    #             attributes = y['attributes']
-    #             for val in important_values:
-    #                 print(attributes[val])
-    #         if links.get('next', False):
-    #             url = links['next']['href']
-    #         else:
-    #             next_page = False
-    #         counter += 1
-    #         print(counter)
-    #         if self.limit > 0:
-    #             if counter >= self.limit:
-    #                 next_page = False
-    #         time.sleep(self.sleep)
+    def get_cookie(self):
+        return f"{self.session}={self.token}"
 
     def __init__(
         self,
+        out_dir=CUR_LOC,
         endpoint=FRD_API,
         user=FRD_USER,
         pw=FRD_PW,
+        session=FRD_SESSION,
+        token=FRD_TOKEN,
         page_size=10,
         limit=10,
         sleep=0.5
@@ -124,19 +104,106 @@ class FrdClient():
         self.endpoint = endpoint
         self.user = user
         self.pw = pw
+        self.session = session
+        self.token = token
+        self.cookie = self.get_cookie()
         self.page_size = page_size
         self.limit = limit
         self.sleep = sleep
+        self.werk_ep = f"{self.endpoint}node/werk"
+        self.manifestation_ep = f"{self.endpoint}node/manifestation"
         self.nsmap = {
             "tei": "http://www.tei-c.org/ns/1.0",
             "xml": "http://www.w3.org/XML/1998/namespace",
         }
-        if self.pw and self.user:
+        if self.pw and self.user or self.session and self.token:
             self.authenticated = True
         else:
             print("no user and password set")
             self.authenticated = False
         self.tei_dummy = self.tei_dummy()
+        self.out_dir = out_dir
+
+
+class FrdWerk(FrdClient):
+    """class to deal with Werke
+    :param werk_id: The hash ID of a Werk Node
+    :type werk_id: str
+
+    :return: A FrdWork instance
+    :rtype: class:`freud_api_crawler.freud_api_crawler.FrdWerk`
+    """
+
+    def get_werk(self):
+        """ returns the werk json as python dict
+
+        :return: a Werk representation
+        :rtrype: dict
+        """
+        url = f"{self.werk_ep}/{self.werk_id}"
+        r = requests.get(
+            self.ep,
+            auth=(self.user, self.pw),
+            headers={
+                'Cookie': self.cookie
+            }
+        )
+        status_code = r.status_code
+        result = r.json()
+        return result
+
+    def get_manifestations(self):
+        man_col = []
+        url = f"{self.manifestation_ep}?filter[field_werk.id]={self.werk_id}&fields[node--manifestation]=id,title"
+        next_page = True
+        while next_page:
+            print(url)
+            response = None
+            result = None
+            x = None
+            response = requests.get(
+                url,
+                auth=(self.user, self.pw),
+                headers={
+                    'Cookie': self.cookie
+                }
+            )
+            result = response.json()
+            links = result['links']
+            if links.get('next', False):
+                url = links['next']['href']
+            else:
+                next_page = False
+            for x in result['data']:
+                item = {
+                    "man_id": x['id'],
+                    "man_title": x['attributes']['title']
+                }
+                man_col.append(item)
+        return man_col
+
+    def __init__(
+        self,
+        werk_id=None,
+        **kwargs
+    ):
+        super().__init__(**kwargs)
+        self.werk_id = werk_id
+        self.ep = f"{self.werk_ep}/{self.werk_id}"
+        self.werk = self.get_werk()
+        self.werk_attrib = self.werk['data']['attributes']
+        for x in self.werk_attrib.keys():
+            value = self.werk_attrib[x]
+            if isinstance(value, dict):
+                for y in value.keys():
+                    dict_key = f"{x}__{y}"
+                    setattr(self, f"md__{dict_key}", value[y])
+            else:
+                setattr(self, f"md__{x}", value)
+        self.meta_attributes = [x for x in dir(self) if x.startswith('md__')]
+        print("fetching related manifestations")
+        self.manifestations = self.get_manifestations()
+        self.manifestations_count = len(self.manifestations)
 
 
 class FrdManifestation(FrdClient):
@@ -153,10 +220,14 @@ class FrdManifestation(FrdClient):
         """ returns the manifest json as python dict
 
         :return: a Manifestation representation
-        :rtrype: dict
+        :rtype: dict
         """
         r = requests.get(
-            self.manifestation_endpoint, auth=(self.user, self.pw)
+            f"{self.manifestation_endpoint}?include=field_werk",
+            auth=(self.user, self.pw),
+            headers={
+                'Cookie': self.cookie
+            }
         )
         status_code = r.status_code
         if status_code != 200:
@@ -170,6 +241,23 @@ class FrdManifestation(FrdClient):
             result = r.json()
         return result
 
+    def get_manifestation_save_path(self):
+        werk_path = self.werk_folder
+        manifestation_path = self.md__path__alias
+        cur_werk_folder = werk_path.split('/')[:3]
+        cur_man_folder = manifestation_path.split('/')[2]
+        man_file_name = f"{manifestation_path.split('/')[-1]}.xml"
+        file_name = os.path.join(
+            self.save_dir, *cur_werk_folder, cur_man_folder, man_file_name
+        )
+        folder = os.path.join(
+            self.save_dir, *cur_werk_folder, cur_man_folder
+        )
+        return {
+            "full_file_name": file_name,
+            "folder": folder
+        }
+
     def get_pages(self):
         """ method returning related page-ids/urls
 
@@ -201,8 +289,14 @@ class FrdManifestation(FrdClient):
         else:
             url = page_id
 
+        print(url)
+
         r = requests.get(
-            url, auth=(self.user, self.pw)
+            f"{url}?include=field_faksimile",
+            auth=(self.user, self.pw),
+            headers={
+                'Cookie': self.cookie
+            }
         )
 
         status_code = r.status_code
@@ -236,15 +330,22 @@ class FrdManifestation(FrdClient):
         body = page_attributes['body']['processed']
         wrapped_body = f'<div xml:id="page__{page_id}">{body}</div>'
         cleaned_body = clean_markup(wrapped_body)
+        faks = page_json['included'][0]
+        page_nr = extract_page_nr(page_attributes['title'])
         result = {
             'id': page_id,
             'title': page_attributes['title'],
+            'page_nr': page_nr,
             'attr': page_attributes,
-            'body': cleaned_body
+            'body': cleaned_body,
+            'faks': faks,
+            'faks__id': faks['id'],
+            'faks__url': faks['links']['self']['href'],
+            'faks__payload': faks['attributes']['uri']['url']
         }
         return result
 
-    def make_xml(self, save=False):
+    def make_xml(self, save=False, limit=True):
 
         """serializes a manifestation as XML/TEI document
 
@@ -261,27 +362,48 @@ class FrdManifestation(FrdClient):
         ] = f"manifestation__{self.manifestation_id}"
         title = doc.xpath('//tei:title[@type="manifestation"]', namespaces=self.nsmap)[0]
         title.text = f"{self.md__title}"
+        w_title = doc.xpath('//tei:title[@type="work"]', namespaces=self.nsmap)[0]
+        w_title.text = f"{self.werk['attributes']['title']}"
         body = doc.xpath('//tei:body', namespaces=self.nsmap)[0]
         pages = self.pages
-        for x in pages[:3]:
+        if limit:
+            actual_pages = pages[:2]
+        else:
+            actual_pages = pages
+        for x in actual_pages:
             page_json = self.get_page(x['id'])
             pp = self.process_page(page_json)
             div = ET.fromstring(pp['body'])
+            pb_el = make_pb(
+                pp['page_nr'],
+                pp['faks__url'],
+                pp['faks__id']
+            )
+            cur_div = div.xpath('//div', namespaces=self.nsmap)[0]
+            cur_div.insert(0, pb_el)
             body.append(div)
         if save:
-            file = f"manifestation__{self.manifestation_id}.xml"
+            try:
+                os.makedirs(self.manifestation_save_location_folder)
+            except FileExistsError:
+                print(f"Overriding exsting file: {self.manifestation_save_location_folder}")
+            file = self.manifestation_save_location_file
             with open(file, 'wb') as f:
                 f.write(ET.tostring(doc, encoding="utf-8"))
         return doc
 
     def __init__(
         self,
-        manifestation_id=None
+        manifestation_id=None,
+        **kwargs
     ):
-        super().__init__()
+        super().__init__(**kwargs)
         self.manifestation_id = manifestation_id
         self.manifestation_endpoint = f"{self.endpoint}node/manifestation/{manifestation_id}"
         self.manifestation = self.get_manifest()
+        self.werk = self.manifestation['included'][0]
+        self.werk_folder = self.werk['attributes']['path']['alias']
+        # self.manifestation_folder = self.manifestation['attributes']['path']['alias']
         self.man_attrib = self.manifestation['data']['attributes']
         for x in self.man_attrib.keys():
             value = self.man_attrib[x]
@@ -294,3 +416,6 @@ class FrdManifestation(FrdClient):
         self.meta_attributes = [x for x in dir(self) if x.startswith('md__')]
         self.pages = self.get_pages()
         self.page_count = len(self.pages)
+        self.save_dir = os.path.join(self.out_dir)
+        self.manifestation_save_location_file = self.get_manifestation_save_path()['full_file_name']
+        self.manifestation_save_location_folder = self.get_manifestation_save_path()['folder']
